﻿Imports Inventor

Public Class Form1
    Private Sub btnBrowse_Click(sender As Object, e As EventArgs) Handles btnBrowse.Click
        Dim fd As OpenFileDialog = New OpenFileDialog()
        Dim fileName As String

        fd.Title = "Select part file"
        fd.Filter = "Part files (*.ipt)|*.ipt"
        fd.FilterIndex = 1
        If fd.ShowDialog() = DialogResult.OK Then
            fileName = fd.FileName
            txtFilename.Text = fileName
        End If
    End Sub

    Private Sub btnGetThreads_Click(sender As Object, e As EventArgs) Handles btnGetThreads.Click
        If rdoApprentice.Checked Then
            Dim apprentice As New Inventor.ApprenticeServerComponent
            Dim apprenticeDoc As ApprenticeServerDocument = Nothing
            Try
                apprenticeDoc = apprentice.Open(txtFilename.Text)
            Catch ex As Exception
                MsgBox("Failed to open the file """ & txtFilename.Text & """")
                Return
            End Try

            GetThreads(apprenticeDoc.ComponentDefinition)
            apprenticeDoc.Close()
            apprentice = Nothing
        Else
            Dim invApp As Inventor.Application
            invApp = GetObject(, "Inventor.Application")
            Dim doc As Inventor.PartDocument = invApp.ActiveDocument
            GetThreads(doc.ComponentDefinition)
        End If
    End Sub

    Public Sub GetThreads(compDef As ComponentDefinition)
        lstResults.Items.Clear()

        ' Iterate through all of the faces in the first body of the part.
        Dim fc As Face
        For Each fc In compDef.SurfaceBodies.Item(1).Faces
            ' Look for cylindrical or conical faces.
            If fc.SurfaceType = SurfaceTypeEnum.kCylinderSurface Or fc.SurfaceType = SurfaceTypeEnum.kConeSurface Then
                If Not fc.ThreadInfos Is Nothing Then
                    ' Check to see if there are any threads on this face.
                    If fc.ThreadInfos.Count > 0 Then
                        ' Get the information for this thread.
                        Dim thread As ThreadInfo
                        For Each thread In fc.ThreadInfos
                            lstResults.Items.Add("Thread")
                            WriteFaceInfo(fc)
                            WriteThreadInfo(fc, thread)
                        Next
                    End If
                End If
            End If
        Next
    End Sub

    ' Write out geometric information about the cylindrical or conical face.
    Private Sub WriteFaceInfo(threadFace As Face)
        If threadFace.SurfaceType = SurfaceTypeEnum.kCylinderSurface Then
            Dim cyl As Cylinder = threadFace.Geometry
            lstResults.Items.Add("   Cylinder Geometry")
            lstResults.Items.Add("      Cylinder Diameter: " & (cyl.Radius * 2).ToString("F5"))
            lstResults.Items.Add("      Cylinder Position: " & GetPointOrVector(cyl.BasePoint))
            lstResults.Items.Add("      Cylinder Direction: " & GetPointOrVector(cyl.AxisVector))
        ElseIf threadFace.SurfaceType = SurfaceTypeEnum.kConeSurface Then
            Dim cn As Cone = threadFace.Geometry
            lstResults.Items.Add("   Cone Geometry")
            lstResults.Items.Add("      Cone Diameter: " & (cn.Radius * 2).ToString("F5"))
            lstResults.Items.Add("      Cone Position: " & GetPointOrVector(cn.BasePoint))
            lstResults.Items.Add("      Cone Direction: " & GetPointOrVector(cn.AxisVector))
            lstResults.Items.Add("      Cone Angle: " & (cn.HalfAngle * (180 / 3.14159265358979)).ToString("F3"))
        End If
    End Sub

    ' Write out all of the information provided by a ThreadInfo object.
    Private Sub WriteThreadInfo(fc As Face, thread As ThreadInfo)
        ' Write out the common, generic thread information.
        lstResults.Items.Add("   General Thread Info")
        lstResults.Items.Add("      Designation: " & thread.ThreadDesignation)
        lstResults.Items.Add("      Custom Designation: " & thread.CustomThreadDesignation)
        lstResults.Items.Add("      Full thread depth: " & thread.FullThreadDepth)
        lstResults.Items.Add("      Internal: " & thread.Internal)
        lstResults.Items.Add("      Metric: " & thread.Metric)
        lstResults.Items.Add("      RightHanded: " & thread.RightHanded)
        lstResults.Items.Add("      Thread Type: " & thread.ThreadType)
        lstResults.Items.Add("      Thread Type Identifier: " & thread.ThreadTypeIdentifier)
        lstResults.Items.Add("      Direction: " & GetPointOrVector(thread.ThreadDirection.AsUnitVector))
        If Not thread.FullThreadDepth Then
            lstResults.Items.Add("      Depth: " & thread.ThreadDirection.Length.ToString("F5"))
        End If

        lstResults.Items.Add("      Locations:")
        Dim basePoint As Point
        For Each basePoint In thread.ThreadBasePoints
            lstResults.Items.Add("         Position: " & GetPointOrVector(basePoint))
        Next

        ' If a tapped hole is created using a sketch so that multiples holes are placed as
        ' a single hole feature, the ThreadBasePoints property will return the location of
        ' all of the holes.  However, there's a problem because each of the physical holes
        ' created by the feature will return this set of points. This code is used to find
        ' the coordinate that is used for this specific physical hole.
        If thread.ThreadBasePoints.Count > 1 Then
            ' Create a transient line along the axis of the cylinder or cone.
            Dim app As Inventor.Application = fc.Application
            Dim fcLine As Line
            fcLine = app.TransientGeometry.CreateLine(fc.Geometry.basePoint, fc.Geometry.AxisVector.AsVector)

            ' Check to see which hole base point lies on the axis.
            Dim foundBasePoint As Point = Nothing
            For Each basePoint In thread.ThreadBasePoints
                Dim dist As Double
                dist = app.MeasureTools.GetMinimumDistance(basePoint, fcLine)
                If dist < 0.000001 Then
                    foundBasePoint = basePoint
                    Exit For
                End If
            Next

            lstResults.Items.Add("         Position Used: " & GetPointOrVector(foundBasePoint))
        End If

        If TypeOf thread Is StandardThreadInfo Then
            ' Print out information for standard threads.
            lstResults.Items.Add("   Standard Thread Info")

            Dim standard As StandardThreadInfo = thread
            lstResults.Items.Add("      Class: " & standard.Class)
            If standard.MajorDiameterMax Then
                lstResults.Items.Add("      Max Major Dia.: " & CType(standard.MajorDiameterMax, Double).ToString("F5"))
            End If
            lstResults.Items.Add("      Min Major Dia.: " & CType(standard.MajorDiameterMin, Double).ToString("F5"))
            lstResults.Items.Add("      Max Minor Dia.: " & CType(standard.MinorDiameterMax, Double).ToString("F5"))
            lstResults.Items.Add("      Min Minor Dia.: " & CType(standard.MinorDiameterMin, Double).ToString("F5"))
            lstResults.Items.Add("      Nominal Size: " & standard.NominalSize)
            lstResults.Items.Add("      Pitch: " & CType(standard.Pitch, Double).ToString("F5"))
            lstResults.Items.Add("      Max Pitch Dia.: " & CType(standard.PitchDiameterMax, Double).ToString("F5"))
            lstResults.Items.Add("      Min Pitch Dia.: " & CType(standard.PitchDiameterMin, Double).ToString("F5"))
            lstResults.Items.Add("      Tap Drill Dia.: " & CType(standard.TapDrillDiameter, Double).ToString("F5"))
        ElseIf TypeOf thread Is TaperedThreadInfo Then
            ' Print out information for tapered threads.
            lstResults.Items.Add("   Tapered Thread Info")

            Dim tapered As TaperedThreadInfo = thread
            lstResults.Items.Add("      Basic Minor Dia.: " & CType(tapered.BasicMinorDiameter, Double).ToString("F5"))
            lstResults.Items.Add("      Effective Dia.: " & CType(tapered.EffectiveDiameter, Double).ToString("F5"))
            lstResults.Items.Add("      Effective Length: " & CType(tapered.EffectiveLength, Double).ToString("F5"))
            lstResults.Items.Add("      Engagement Dia.: " & CType(tapered.EngagementDiameter, Double).ToString("F5"))
            lstResults.Items.Add("      Engagement Length: " & CType(tapered.EngagementLength, Double).ToString("F5"))
            lstResults.Items.Add("      External Pitch Dia.: " & CType(tapered.ExternalPitchDiameter, Double).ToString("F5"))
            lstResults.Items.Add("      External Nominal Dia.: " & CType(tapered.NominalExternalDiameter, Double).ToString("F5"))
            lstResults.Items.Add("      External Nomminal Length: " & CType(tapered.NominalExternalLength, Double).ToString("F5"))
            lstResults.Items.Add("      Outside Pipe Dia.:   " & CType(tapered.OutsidePipeDiameter, Double).ToString("F5"))
            lstResults.Items.Add("      Overall External Length: " & CType(tapered.OverallExternalLength, Double).ToString("F5"))
            lstResults.Items.Add("      Tap Drill Dia.:   " & CType(tapered.TapDrillDiameter, Double).ToString("F5"))
            lstResults.Items.Add("      Thread Height: " & CType(tapered.ThreadHeight, Double).ToString("F5"))
            lstResults.Items.Add("      Threads Per Inch:   " & tapered.ThreadsPerInch)
            lstResults.Items.Add("      Vanish Thread: " & CType(tapered.VanishThread, Double).ToString("F5"))
            lstResults.Items.Add("      Wrench Makeup Dia.:   " & CType(tapered.WrenchMakeupDiameter, Double).ToString("F5"))
            lstResults.Items.Add("      Wrench Makeup Length: " & CType(tapered.WrenchMakeupLength, Double).ToString("F5"))
        End If
    End Sub

    ' Function used to create a printable string from a point or vector.
    Private Function GetPointOrVector(pointOrVector As Object) As String
        Dim x As Double = pointOrVector.X
        Dim y As Double = pointOrVector.Y
        Dim z As Double = pointOrVector.Z
        Return (x.ToString("F5") & ", " &
                y.ToString("F5") & ", " &
                z.ToString("F5"))
    End Function

    Private Sub rdoInventor_CheckedChanged(sender As Object, e As EventArgs) Handles rdoInventor.CheckedChanged
        If rdoInventor.Checked Then
            txtFilename.Enabled = False
            btnBrowse.Enabled = False
        End If
    End Sub

    Private Sub rdoApprentice_CheckedChanged(sender As Object, e As EventArgs) Handles rdoApprentice.CheckedChanged
        If rdoApprentice.Checked Then
            txtFilename.Enabled = True
            btnBrowse.Enabled = True
        End If
    End Sub
End Class
